import 'dart:async';
import 'dart:ui';

class Cancelable<O> implements Future {
  final Completer<O> _completer;
  final VoidCallback onCancel;

  Cancelable(this._completer, this.onCancel);

  void cancel() {
    onCancel();
    if (!_completer.isCompleted) _completer.completeError(CanceledError());
  }

  @override
  Stream asStream() => _completer.future.asStream();

  @override
  Future catchError(Function onError, {bool Function(Object error) test}) =>
      _completer.future.catchError(onError, test: test);

  @override
  Future<R> then<R>(Function(O value) onValue, {Function onError}) =>
      _completer.future.then(onValue).catchError(onError);

  @override
  Future timeout(Duration timeLimit, {FutureOr Function() onTimeout}) => _completer.future.timeout(timeLimit);

  @override
  Future whenComplete(FutureOr Function() action) => _completer.future.whenComplete(action);
}

class CanceledError implements Exception {}
